/*===========================================================================

   SPW Main thing

   $ Version 1.0  21/12/94  22:11:21 $

   Copyright 1994 by Marco Savegnago.  All rights reserved.
   This software may be freely distributed and used, but it may not
   under any circumstances be sold by anyone other than the author.
   It may be distributed by a commercial company as long as it is
   for no cost.

===========================================================================*/

//  Local include

#include "spw.h"
#include "md2.h"
#include "matrix.h"

//  OWL Include

#include <owl\clipboar.h>

//  CRTL    include

#include <string.h>
#include <cstring.h>

//  Resource ID include

#include "spw.rh"

// For diagnostics
// DIAG_DEFINE_GROUP(TWinPass,1,0);

DEFINE_RESPONSE_TABLE1(TComboEdit,TEdit)
  EV_WM_CHAR,
END_RESPONSE_TABLE;

//
// This is where we look for the <ENTER> key and execute commands
//
void TComboEdit::EvChar(UINT key, UINT /*repeatCount*/, UINT /*flags*/)
{
	if (key == VK_RETURN)
	{
		//
		// Get Line into local buffer for parsing
		//

		GetText(EditBuf, sizeof(EditBuf));

		if (!EditBuf[0]) // Ignore empty strings
					return;

		ExecuteCmd(EditBuf);

		return;
	}
	else
		DefaultProcessing();

	return;
}

void TComboEdit::ExecuteCmd(const char* cmd)
{
	if (cmd)
	{
        //  Get application configuration

        TConfigData* config = &((TWinPass *)GetApplication())->config;

        string  rpl;

        if(config->MD2Password)
        {
            string  challenge = cmd;

            challenge += config->passwordString;

		    MD2 md2(challenge.c_str());

            rpl =   md2.getString();
        }
        else
        {
            TMatrix matrix(5, config->passwordString);

            matrix.makeReplay(cmd);

            rpl =   matrix.getString();
        }

		MessageBox(rpl.c_str(), "Result", MB_OK);

        //  Has the result to be copied to the clipboard ?

        if(config->copyToClipboard)
            copyToClipboard(rpl.c_str());

		Combo->InsertString(EditBuf,0);

		Clear();

		return;
	}
}

/*
    Copy the string to the clipboard as text
*/

void TComboEdit::copyToClipboard(const char *str)
{
    HANDLE  StringGlobalHandle;
    LPSTR   StringGlobalPtr;

    //  Allocate the buffer

    StringGlobalHandle = GlobalAlloc (GMEM_MOVEABLE, (LONG)lstrlen(str) + 1);

    if (StringGlobalHandle)
    {
        //  If it is OK the fix it

     	StringGlobalPtr = (char*)GlobalLock(StringGlobalHandle);

     	if (StringGlobalPtr)
        {
            //  Copy the resulting string

       		lstrcpy(StringGlobalPtr, str);
       		GlobalUnlock(StringGlobalHandle);

            TClipboard  clipboard(HWindow);

            //  The copy to the clipboard

       		if (clipboard)
            {
        		clipboard.EmptyClipboard();
        		clipboard.SetClipboardData(CF_TEXT, StringGlobalHandle);
        		clipboard.CloseClipboard();
        		return;
            }
       		else
				GlobalFree(StringGlobalHandle);
     	}
     	else
			GlobalFree(StringGlobalHandle);
    }

    return;
}


//------------------------------------------------------------------------

TLineEdit::TLineEdit(TWindow* p, int w, int h)
:
	TComboBox(p, COMBO_ID, -1, -1, w, h, CBS_DROPDOWN, CMDBUFSIZE-1)
{
	Ed = new TComboEdit(this, CMDBUFSIZE-1);

  // We don't want a sorted list!
	Attr.Style &= ~CBS_SORT;
}
//------------------------------------------------------------------------

DEFINE_RESPONSE_TABLE1(TMakePassFrame, TFloatingFrame)
EV_COMMAND(CM_CONFIG, CmConfig),
EV_COMMAND(CM_ABOUT,  CmAbout),
END_RESPONSE_TABLE;

TMakePassFrame::TMakePassFrame(TWindow* client, char* title)
:
	TFloatingFrame(0, title, client)
{
   //
   // We can set our own attributes, but must be before EnableTiny...
   //
   Attr.Style = WS_POPUP | WS_BORDER | WS_SYSMENU | WS_MINIMIZEBOX;

   //
   // Or we can be specific, even make it BIGGER if we want
   // If we set the option CloseBox flag to true, then clicking the box
   // will close the window instead of displaying the system menu
   //
   EnableTinyCaption(52, FALSE);
   //
   // Create our combo box and size it for an edit line and 5 text items
   // What's the correct way to get the height of an edit control?
   //
   int lineH = GetSystemMetrics(SM_CYCAPTION)+2; // 1pixel top & bottom border

   Cmd = new TLineEdit(this, EDITWIDTH, lineH*6);

   Attr.X = (GetSystemMetrics(SM_CXSCREEN) - EDITWIDTH)/2;
   Attr.Y = (GetSystemMetrics(SM_CYSCREEN) - lineH*6)/2;
   Attr.W = EDITWIDTH;
   Attr.H = 0;  // Resize ourselves later, when we know the actual size
}

void TMakePassFrame::SetupWindow()
{
#ifdef  __USEDDE
    TWinPass::This = this;
#endif

	TFloatingFrame::SetupWindow();

	// Append about menu item to system menu.
	//

	TSystemMenu sysMenu(HWindow);
	sysMenu.AppendMenu(MF_SEPARATOR, 0, (LPSTR)0);
	sysMenu.AppendMenu(MF_STRING, CM_CONFIG, "Configure...");
	sysMenu.AppendMenu(MF_SEPARATOR, 0, (LPSTR)0);
	sysMenu.AppendMenu(MF_STRING, CM_ABOUT, "About...");

	//
	// Resize window to actual combo box size
	//
	TRect cr = Cmd->GetWindowRect();
	TRect newRect = GetWindowRect();
	MoveWindow(newRect.left, newRect.top, newRect.Width(), CaptionHeight + cr.Height(), TRUE);

#ifdef  __USEDDE

    Service = Topic = Item = 0;

    // The strings below are the 'Service', 'Topic' and 'Item' identifiers
    // that this application makes available, through DDE, to other
    // applications.
    //

    Service = DdeCreateStringHandle(InstId(), "PKLIB_SERVER", CP_WINANSI);
    Topic   = DdeCreateStringHandle(InstId(), "PACKET_PASSWORD", CP_WINANSI);
    Item    = DdeCreateStringHandle(InstId(), "NMD2_SUPPORT", CP_WINANSI);

    if (Service && Topic && Item)
    {
        if (DdeNameService(InstId(), Service, 0, DNS_REGISTER) == 0)
        {
            MessageBox("DDE Registration failed.", Title, MB_ICONSTOP);
            PostQuitMessage(0);
        }
    }
    else
    {
        MessageBox("String creation failed.", Title, MB_ICONSTOP);
        PostQuitMessage(0);
    }
#endif
}


TMakePassFrame::~TMakePassFrame()
{

#ifdef  __USEDDE

    // This clean up is required for those resources that were allocated during
    // the DDE conversation.
    //
    if (HConv)
        DdeDisconnect(HConv);   // Let the other party know we are leaving

    if (InstId())
    {
        DdeNameService(InstId(), Service, 0, DNS_UNREGISTER);

        if (Service)
            DdeFreeStringHandle(InstId(), Service);
        if (Topic)
            DdeFreeStringHandle(InstId(), Topic);
        if (Item)
            DdeFreeStringHandle(InstId(), Item);
    }
#endif
}

#ifdef  __USEDDE

//
// This function is used to compare incoming Topic and Service requests.
// This example DDE Server only makes one Service and one Topic available
// so the logic is simple for this case but could be more complex for
// Servers that offer multiple Services or Topics.
//

BOOL        TMakePassFrame::MatchTopicAndService(HSZ topic, HSZ service)
{
    if (DdeCmpStringHandles(Topic, topic) == 0)
        if (DdeCmpStringHandles(Service, service) == 0)
                                            return TRUE;

    return FALSE;
}

//
// This function is used to compare incoming Topic and Item pair requests.
// This Server only makes one Topic with one Item available so the logic is
// simple for this case but could be more complex if the Server offered
// multiple Items for multiple Topics.
//

BOOL        TMakePassFrame::MatchTopicAndItem(HSZ hsz1, HSZ hsz2)
{
    if (DdeCmpStringHandles(Topic, hsz1) == 0)
        if (DdeCmpStringHandles(Item, hsz2) == 0)
                                        return TRUE;

    return FALSE;

}

//
// This function responds to 'system wide' polling of any available
// Services with specific Topics, any Topics with specific Services or any
// Services with any Topics.  It simply replies with a match, if there is
// one, so that the polling application can open data discussions later, if
// desired.
//

HDDEDATA    TMakePassFrame::WildConnect(HSZ hsz1, HSZ hsz2, WORD wFmt)
{
    HSZPAIR hszpTemp[] = { { Service, Topic }, { 0, 0 } };

    if (!hsz1 && !hsz2)    // Returns all if true
        return DdeCreateDataHandle(InstId(), (LPBYTE)&hszpTemp[0], sizeof(hszpTemp), 0L, 0, wFmt, 0);

    if (!hsz1 && DdeCmpStringHandles(hsz2, Service) == 0)
        return DdeCreateDataHandle(InstId(), (LPBYTE)&hszpTemp[0], sizeof(hszpTemp), 0L, 0, wFmt, 0);

    if (DdeCmpStringHandles(hsz1, Topic) == 0 && !hsz2)
        return DdeCreateDataHandle(InstId(), (LPBYTE)&hszpTemp[0], sizeof(hszpTemp), 0L, 0, wFmt, 0);

   return 0;
}

DWORD   TMakePassFrame::InstId()
{
    return ((TWinPass*)GetApplication())->InstId;
}

HDDEDATA    TMakePassFrame::DataRequested(WORD)
{
  return 0;
}

void        TMakePassFrame::UpdateData()
{
    DdePostAdvise(InstId(), Topic, Item);
}
#endif

/*
    Execute configuration dialog
*/

void TMakePassFrame::CmConfig()
{
    TConfigData* config = &((TWinPass *)GetApplication())->config;

	 TConfigDialog   configDialog(this, IDD_CONFIG_DIALOG, config);

    configDialog.Execute();
}

/*
    User function that reply to About Menu Item
*/

void TMakePassFrame::CmAbout()
{
    TDialog aboutDialog(this, IDD_AUTORE_DIALOG);

    aboutDialog.Execute();
}


//  Static

const char *        TWinPass::ApplicationName = "SPW";
char                TWinPass::ProfileName[MAXPATH];

#ifdef  __USEDDE
TMakePassFrame*     TWinPass::This;
#endif

/*
    Application ctor
*/

TWinPass::TWinPass()
:
    TApplication(ApplicationName)
#ifdef  __USEDDE
    ,CallBackProc((FARPROC)CallBack)
#endif
{
    char    buffer[256];

/*******************************************************************
	Get the default ini filename from the resource
*******************************************************************/

	 if(ProfileName[0] == NULL)
		 LoadString(IDS_INIFILENAME,ProfileName, MAXPATH);

/*******************************************************************
	Get PasswordType item from the INI file (default is MD2)
*******************************************************************/

    GetPrivateProfileString(ApplicationName,"PasswordType","MD2",buffer,sizeof(buffer),ProfileName);

/*******************************************************************
	Check the value
*******************************************************************/

    if(!stricmp(buffer,"MD2"))
    {
        config.MD2Password = TRUE;
        config.MatrixPassword = FALSE;
	}
    else
    {
        config.MD2Password = FALSE;
        config.MatrixPassword = TRUE;
	}

/*******************************************************************
	Get Matrix Len from the INI file
*******************************************************************/

    int len = GetPrivateProfileInt(ApplicationName,"MatrixLen",5,ProfileName);

    itoa(len, config.matrixLen, 10);

/*******************************************************************
	Get PasswordString item from the INI file
*******************************************************************/

    GetPrivateProfileString(ApplicationName,"PasswordString","PUT HERE YOUR PASSWORD",buffer,sizeof(buffer),ProfileName);

    strcpy(config.passwordString, buffer);

/*******************************************************************
	Get CopyToClipboard item from the INI file
*******************************************************************/

    config.copyToClipboard = BOOL(GetPrivateProfileInt(ApplicationName,"CopyToClipboard",1,ProfileName));
}

/*
    Application dtor
*/

TWinPass::~TWinPass()
{
    char buff[10];

    WritePrivateProfileString(ApplicationName,"PasswordType",config.MD2Password ? "MD2" : "MATRIX", ProfileName);
    WritePrivateProfileString(ApplicationName,"MatrixLen",config.matrixLen , ProfileName);
    WritePrivateProfileString(ApplicationName,"PasswordString",config.passwordString, ProfileName);

    itoa(config.copyToClipboard,buff,10);

    WritePrivateProfileString(ApplicationName,"CopyToClipboard",buff, ProfileName);
}

void TWinPass::InitMainWindow()
{
	 // EnableCtl3d();
    EnableBWCC();
	 MainWindow = new TMakePassFrame(0,"WinPass");
    MainWindow->SetIcon(this, IDI_SPW_APPLICATION);
}

#ifdef  __USEDDE

//
// This call back function is the heart of interaction between this program
// and DDE.  Because Windows doesn't pass C++ 'this' pointers to call
// back functions, a static 'this' pointer was used.  If you wanted to
// create a Server that would allow for more than one conversation, using a
// List of conversations and their associated 'this' pointers would be one
// possible method to try.  The XTYP_ constants are described in detail in
// the online help.
//
HDDEDATA FAR PASCAL TWinPass::CallBack(WORD wType, WORD wFmt, HCONV hConv, HSZ hsz1, HSZ hsz2, HDDEDATA hData, DWORD, DWORD)
{
    switch (wType)
    {
        case XTYP_ADVREQ:
           if (This->MatchTopicAndItem(hsz1, hsz2))
               return This->DataRequested(wFmt);

           return 0;

        case XTYP_CONNECT:
            if (!This->HConv)
                if (This->MatchTopicAndService(hsz1, hsz2))
                                            return (HDDEDATA)1;

            return 0;

        case XTYP_CONNECT_CONFIRM:
            This->HConv = hConv;
            break;

        case XTYP_DISCONNECT:
            if (hConv == This->HConv)
            {
                This->HConv = 0;
            }
            break;

        case XTYP_ERROR:
            This->MessageBox("A critical DDE error has occured.", This->Title, MB_ICONINFORMATION);
            break;

        case XTYP_EXECUTE:
            // if (This->MatchTopicAndItem(hsz1, hsz2))

            return DDE_FNOTPROCESSED;

        case XTYP_POKE:
        {
            char temp[128];
            strcpy(temp, "The server received: ");
            int size = strlen(temp);
            DdeGetData(hData, (unsigned char *)&temp[size], sizeof(temp) - size, 0);
            ::MessageBox(GetFocus(), temp, This->Title, MB_ICONINFORMATION);
            return (HDDEDATA)DDE_FACK;
        }

        case XTYP_REQUEST:
            if (This->MatchTopicAndItem(hsz1, hsz2))
                return This->DataRequested(wFmt);
            return 0;

        case XTYP_WILDCONNECT:
            return This->WildConnect(hsz1, hsz2, wFmt);
   }

    return 0;
}

#endif

void TWinPass::InitInstance()
{
  // The code below sets up the DDEML call back function that is used by the
  // DDE Management Library to carry out data transfers between
  // applications.
  //
#ifdef  __USEDDE

   if (DdeInitialize(&InstId, (PFNCALLBACK)(FARPROC)CallBackProc, 0, 0) != DMLERR_NO_ERROR)
   {
      ::MessageBox(0,"Initialization failed.", "DDEML Server", MB_ICONSTOP|MB_TASKMODAL);
      PostQuitMessage(0);
   }
#endif
  // Must come after we've initialized DDE since InitInstance will trigger
  // SetupWindow
    TApplication::InitInstance();
}

int TWinPass::TermInstance(int status)
{
#ifdef  __USEDDE
    if (InstId)
    {
        DdeUninitialize(InstId);
    }
#endif

    return TApplication::TermInstance(status);
}


//  Main ...


int OwlMain(int argc, char* argv[])
{
    if (argc > 1)
        for(int i=1;i<argc;i++)
        {
            char *p=argv[i];
            if(*p=='-'||*p=='/')
            {
                p++;
                switch(toupper(*p))
                {
                    case 'C':      // /C specifica un file di configurazione alternativo
                        if (*++p)
                            strcpy(TWinPass::ProfileName, p);
                        break;

                    default:
                        break;
                }
			}
         }

    return TWinPass().Run();
}
